@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/GraphicsView/include/CGAL/Buffer_for_vao.h $
-// $Id: Buffer_for_vao.h 2cbc381 2021-09-23T16:13:57+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/GraphicsView/include/CGAL/Buffer_for_vao.h $
+// $Id: Buffer_for_vao.h 4547818 2022-11-15T13:39:40+01:00 albert-github
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -23,6 +23,7 @@
 #include <CGAL/Constrained_triangulation_plus_2.h>
 #include <CGAL/Cartesian_converter.h>
 #include <CGAL/IO/Color.h>
+#include <CGAL/assertions.h>
 
 #include <vector>
 #include <cstdlib>
@@ -59,7 +60,7 @@ namespace internal
       ++nb;
     }
 
-    assert(nb>0);
+    CGAL_assertion(nb>0);
     return (typename Kernel_traits<Vector>::Kernel::Construct_scaled_vector_3()
             (normal, 1.0/nb));
   }
@@ -106,7 +107,7 @@ namespace internal
     }
   };
 
-  // Specialization when K==Local_kernel, because there is no need of convertion here.
+  // Specialization when K==Local_kernel, because there is no need of conversion here.
   template<typename Local_kernel>
   struct Geom_utils<Local_kernel, Local_kernel>
   {
@@ -577,7 +578,7 @@ protected:
           add_gouraud_normal(m_vertex_normals_for_face[i]);
         }
         else
-        { // Here user does not provide all vertex normals: we use face normal istead
+        { // Here user does not provide all vertex normals: we use face normal instead
           // and thus we will not be able to use Gouraud
           add_gouraud_normal(normal);
         }
@@ -703,7 +704,7 @@ protected:
         else { ++(edges[p1][p2]); }
       }
 
-      // (1) We insert all the edges as contraint in the CDT.
+      // (1) We insert all the edges as constraint in the CDT.
       typename CDT::Vertex_handle previous=nullptr, first=nullptr;
       for (unsigned int i=0; i<m_points_of_face.size(); ++i)
       {
@@ -872,7 +873,7 @@ protected:
   }
 
 protected:
-  // Types usefull for triangulation
+  // Types useful for triangulation
   struct Vertex_info
   {
     Local_vector v;
@@ -923,3 +924,4 @@ protected:
 } // End namespace CGAL
 
 #endif // CGAL_VBO_BUFFER_FILLER_H
+
