@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/Triangulation_on_sphere_2/include/CGAL/Delaunay_triangulation_on_sphere_2.h $
-// $Id: Delaunay_triangulation_on_sphere_2.h d871728 2021-07-23T13:09:37+02:00 Mael Rouxel-Labbé
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/Triangulation_on_sphere_2/include/CGAL/Delaunay_triangulation_on_sphere_2.h $
+// $Id: Delaunay_triangulation_on_sphere_2.h f15b144 2023-06-05T15:10:11+02:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Mariette Yvinec,
@@ -20,7 +20,7 @@
 #include <CGAL/Triangulation_on_sphere_2.h>
 #include <CGAL/Triangulation_on_sphere_face_base_2.h>
 #include <CGAL/Triangulation_on_sphere_vertex_base_2.h>
-#include <CGAL/triangulation_assertions.h>
+#include <CGAL/assertions.h>
 
 #include <CGAL/enum.h>
 #include <CGAL/Origin.h>
@@ -28,7 +28,6 @@
 #include <CGAL/spatial_sort_on_sphere.h>
 #include <CGAL/Spatial_sort_traits_adapter_3.h>
 
-#include <boost/iterator/transform_iterator.hpp>
 #include <boost/property_map/function_property_map.hpp>
 
 #include <algorithm>
@@ -148,7 +147,7 @@ public:
   {
   }
 
-  // Assignement
+  // Assignment
   Delaunay_triangulation_on_sphere_2& operator=(Delaunay_triangulation_on_sphere_2 other) // intentional copy
   {
     Base::swap(static_cast<Base&>(other));
@@ -260,9 +259,9 @@ public:
   template <typename P>
   Vertex_handle insert(const P& p,
                        Face_handle f = Face_handle(),
-                       typename std::enable_if<!std::is_same<P, Point>::value>::type* = nullptr)
+                       std::enable_if_t<!std::is_same<P, Point>::value>* = nullptr)
   {
-    CGAL_triangulation_assertion((std::is_same<P, Point_3>::value));
+    CGAL_assertion((std::is_same<P, Point_3>::value));
 
     return insert(geom_traits().construct_point_on_sphere_2_object()(p), f);
   }
@@ -292,16 +291,16 @@ public:
   // Input range has value type Point, with Point != Point_3
   template <typename InputIterator>
   size_type insert(InputIterator first, InputIterator beyond,
-                   typename std::enable_if<
+                   std::enable_if_t<
                               !std::is_same<typename std::iterator_traits<InputIterator>::value_type,
-                                            Point_3>::value>::type* = nullptr);
+                                            Point_3>::value>* = nullptr);
 
   // Input range has value type Point_3, possibly with Point == Point_3
   template <typename InputIterator>
   size_type insert(InputIterator first, InputIterator beyond,
-                   typename std::enable_if<
+                   std::enable_if_t<
                               std::is_same<typename std::iterator_traits<InputIterator>::value_type,
-                                           Point_3>::value>::type* = nullptr);
+                                           Point_3>::value>* = nullptr);
 
   bool update_ghost_faces(Vertex_handle v, bool first = false);
 
@@ -475,7 +474,7 @@ typename Triangulation_on_sphere_2<Gt, Tds>::Vertex_handle
 Delaunay_triangulation_on_sphere_2<Gt, Tds>::
 insert_third(const Point& p)
 {
-  CGAL_triangulation_assertion(number_of_vertices() == 2);
+  CGAL_assertion(number_of_vertices() == 2);
 
   Vertex_handle v = vertices_begin();
   Vertex_handle u = v->face()->neighbor(0)->vertex(0);
@@ -494,10 +493,10 @@ insert_third(const Point& p)
 
   nv->set_point(p);
 
-  CGAL_triangulation_assertion_code(Face_handle f = all_edges_begin()->first;)
-  CGAL_triangulation_assertion(orientation_on_sphere(point(f, 0),
-                                                     point(f, 1),
-                                                     point(f->neighbor(0), 1)) != RIGHT_TURN);
+  CGAL_assertion_code(Face_handle f = all_edges_begin()->first;)
+  CGAL_assertion(orientation_on_sphere(point(f, 0),
+                                       point(f, 1),
+                                       point(f->neighbor(0), 1)) != RIGHT_TURN);
 
   return nv;
 }
@@ -517,7 +516,7 @@ insert_outside_affine_hull_regular(const Point& p)
   const Point& p1 = point(f, 1);
   const Point& p2 = point(fn, 1);
 
-  CGAL_triangulation_assertion(orientation_on_sphere(p0, p1, p2) != NEGATIVE);
+  CGAL_assertion(orientation_on_sphere(p0, p1, p2) != NEGATIVE);
   Orientation orient2 = side_of_oriented_circle(p0, p1, p2, p);
 
   if(orient2 == POSITIVE)
@@ -588,7 +587,7 @@ insert(const Point& p, Locate_type lt, Face_handle loc, int /*li*/)
     }
   }
 
-  CGAL_triangulation_assertion(false);
+  CGAL_assertion(false);
   return v;
 }
 
@@ -607,7 +606,7 @@ insert(const Point& p, Face_handle start)
       return Vertex_handle();
     case TOO_CLOSE:
     {
-      CGAL_triangulation_assertion(loc != Face_handle());
+      CGAL_assertion(loc != Face_handle());
       return loc->vertex(li);
     }
     case VERTEX:
@@ -627,9 +626,9 @@ template <typename InputIterator>
 typename Delaunay_triangulation_on_sphere_2<Gt, Tds>::size_type
 Delaunay_triangulation_on_sphere_2<Gt, Tds>::
 insert(InputIterator first, InputIterator beyond,
-       typename std::enable_if<
+       std::enable_if_t<
                   !std::is_same<typename std::iterator_traits<InputIterator>::value_type,
-                                Point_3>::value>::type*)
+                                Point_3>::value>*)
 {
   typedef Point_3_with_iterator<Self>                                P3_wit;
 
@@ -662,7 +661,7 @@ insert(InputIterator first, InputIterator beyond,
   points.reserve(input_points.size());
   for(const P3_wit& p3wi : p3_points)
   {
-    CGAL_triangulation_assertion(p3wi.input_point_ptr != nullptr);
+    CGAL_assertion(p3wi.input_point_ptr != nullptr);
     points.push_back(*(p3wi.input_point_ptr));
   }
 
@@ -683,9 +682,9 @@ template <typename InputIterator>
 typename Delaunay_triangulation_on_sphere_2<Gt, Tds>::size_type
 Delaunay_triangulation_on_sphere_2<Gt, Tds>::
 insert(InputIterator first, InputIterator beyond,
-       typename std::enable_if<
+       std::enable_if_t<
                   std::is_same<typename std::iterator_traits<InputIterator>::value_type,
-                               Point_3>::value>::type*)
+                               Point_3>::value>*)
 {
   const size_type n = number_of_vertices();
 
@@ -716,7 +715,7 @@ bool
 Delaunay_triangulation_on_sphere_2<Gt, Tds>::
 update_ghost_faces(Vertex_handle v, bool first)
 {
-  CGAL_triangulation_assertion(dimension() == 2);
+  CGAL_assertion(dimension() == 2);
 
   bool ghost_found = false;
 
@@ -737,7 +736,7 @@ update_ghost_faces(Vertex_handle v, bool first)
   }
   else // not first
   {
-    CGAL_triangulation_assertion(v != Vertex_handle());
+    CGAL_assertion(v != Vertex_handle());
     Face_circulator fc = this->incident_faces(v, v->face());
     Face_circulator done(fc);
     do
@@ -764,7 +763,7 @@ void
 Delaunay_triangulation_on_sphere_2<Gt, Tds>::
 remove_1D(Vertex_handle v)
 {
-  CGAL_triangulation_precondition(v != Vertex_handle());
+  CGAL_precondition(v != Vertex_handle());
 
   tds().remove_1D(v);
 }
@@ -793,7 +792,7 @@ void
 Delaunay_triangulation_on_sphere_2<Gt, Tds>::
 remove(Vertex_handle v)
 {
-  CGAL_triangulation_precondition(v != Vertex_handle());
+  CGAL_precondition(v != Vertex_handle());
 
   if(number_of_vertices() <= 3)
     tds().remove_dim_down(v);
@@ -825,7 +824,7 @@ test_dim_down(Vertex_handle v)
     if(it4 == vertices_end())
       break;
 
-    CGAL_triangulation_assertion(it != v && it2 != v && it3 != v && it4 != v);
+    CGAL_assertion(it != v && it2 != v && it3 != v && it4 != v);
 
     if(side_of_oriented_circle(it->point(), it2->point(), it3->point(), it4->point()) != ON_ORIENTED_BOUNDARY)
       return false;
@@ -1135,7 +1134,7 @@ is_valid_face(Face_handle fh, bool verbose, int /*level*/) const
   {
     Orientation test = side_of_oriented_circle(fh, point(fh->vertex(i)));
     result = result && test == ON_ORIENTED_BOUNDARY;
-    CGAL_triangulation_assertion(result);
+    CGAL_assertion(result);
   }
 
   if(!result)
@@ -1147,7 +1146,7 @@ is_valid_face(Face_handle fh, bool verbose, int /*level*/) const
     }
   }
 
-  CGAL_triangulation_assertion(result);
+  CGAL_assertion(result);
   return result;
 }
 
@@ -1163,7 +1162,7 @@ is_valid(bool verbose, int level) const
     if(verbose)
       Rcpp::Rcerr << "invalid data structure" << std::endl;
 
-    CGAL_triangulation_assertion(false);
+    CGAL_assertion(false);
     return false;
   }
 
@@ -1178,18 +1177,18 @@ is_valid(bool verbose, int level) const
     case 0:
       break;
     case 1:
-      CGAL_triangulation_assertion(this->is_plane());
+      CGAL_assertion(this->is_plane());
       break;
     case 2:
       for(All_faces_iterator it=all_faces_begin(); it!=all_faces_end(); ++it)
       {
         Orientation s = orientation_on_sphere(point(it, 0), point(it, 1), point(it, 2));
         result = result && (s != NEGATIVE || it->is_ghost());
-        CGAL_triangulation_assertion(result);
+        CGAL_assertion(result);
       }
 
       result = result && (number_of_faces() == 2 * number_of_vertices() - 4);
-      CGAL_triangulation_assertion(result);
+      CGAL_assertion(result);
       break;
   }
 
@@ -1197,10 +1196,11 @@ is_valid(bool verbose, int level) const
   if(verbose)
     Rcpp::Rcerr << " number of vertices " << number_of_vertices() << "\t" << std::endl;
 
-  CGAL_triangulation_assertion(result);
+  CGAL_assertion(result);
   return result;
 }
 
 } // namespace CGAL
 
 #endif // CGAL_DELAUNAY_TRIANGULATION_ON_SPHERE_2_H
+
