@@ -6,8 +6,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/Filtered_kernel/include/CGAL/Filtered_kernel/internal/Static_filters/Do_intersect_3.h $
-// $Id: Do_intersect_3.h 381ff61 2021-10-08T21:52:32+02:00 Mael Rouxel-Labbé
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/Filtered_kernel/include/CGAL/Filtered_kernel/internal/Static_filters/Do_intersect_3.h $
+// $Id: Do_intersect_3.h b5c21e1 2022-11-23T18:48:43+01:00 Mael Rouxel-Labbé
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -27,7 +27,7 @@
 
 #include <iostream>
 
-// inspired from http://cag.csail.mit.edu/~amy/papers/box-jgt.pdf
+// inspired from https://people.csail.mit.edu/amy/papers/box-jgt.pdf
 
 namespace CGAL {
 
@@ -510,6 +510,8 @@ public:
             (pts[i][1] >= b.ymin() && pts[i][1] <= b.ymax()) &&
             (pts[i][2] >= b.zmin() && pts[i][2] <= b.zmax()) )
         {
+          // If any of the three points of the triangle is inside the bbox,
+          // then the box and triangle intersect.
           return true;
         }
 
@@ -521,6 +523,21 @@ public:
       }
     }
 
+    // If the bbox of the triangle does not intersect `b`, then the bbox and
+    // the triangle do not intersect.
+    for(int i=0; i< 3; ++i) {
+      double triangle_bbox_min = pts[0][i];
+      double triangle_bbox_max = triangle_bbox_min;
+      for(int j=1; j<3; ++j) {
+        if(pts[j][i] < triangle_bbox_min)
+          triangle_bbox_min = pts[j][i];
+        if(pts[j][i] > triangle_bbox_max)
+          triangle_bbox_max = pts[j][i];
+      }
+      if(triangle_bbox_min > b.max_coord(i) || triangle_bbox_max < b.min_coord(i))
+        return false;
+    }
+
     // copy of the regular code with do_axis_intersect_aux_impl statically filtered
     auto do_axis_intersect_aux_impl = [](double alpha, double beta, double c_alpha, double c_beta) -> Uncertain<Sign>
     {
@@ -841,3 +858,4 @@ public:
 } // end namespace CGAL
 
 #endif  // CGAL_INTERNAL_STATIC_FILTERS_DO_INTERSECT_3_H
+
