@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/Surface_mesh_skeletonization/include/CGAL/Mean_curvature_flow_skeletonization.h $
-// $Id: Mean_curvature_flow_skeletonization.h 936968f 2021-09-06T10:38:24+02:00 Dmitry Anisimov
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/Surface_mesh_skeletonization/include/CGAL/Mean_curvature_flow_skeletonization.h $
+// $Id: Mean_curvature_flow_skeletonization.h 9cc1dda 2023-06-26T16:30:17+01:00 Andreas Fabri
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : Xiang Gao <gaox@ethz.ch>
@@ -20,15 +20,18 @@
 #include <CGAL/Timer.h>
 #include <CGAL/Default.h>
 
-#include <CGAL/Polyhedron_3.h>
-#include <CGAL/Polyhedron_items_with_id_3.h>
-#include <CGAL/boost/graph/graph_traits_Polyhedron_3.h>
+#include <CGAL/HalfedgeDS_default.h>
+#include <CGAL/HalfedgeDS_vertex_max_base_with_id.h>
+#include <CGAL/HalfedgeDS_halfedge_max_base_with_id.h>
+#include <CGAL/HalfedgeDS_face_max_base_with_id.h>
+
+#include <CGAL/boost/graph/graph_traits_HalfedgeDS_default.h>
+#include <CGAL/boost/graph/properties_HalfedgeDS_default.h>
 #include <CGAL/boost/graph/copy_face_graph.h>
 
 #include <boost/graph/graph_traits.hpp>
 #include <boost/graph/adjacency_list.hpp>
 #include <boost/graph/copy.hpp>
-#include <boost/unordered_map.hpp>
 #include <boost/property_map/property_map.hpp>
 
 #include <CGAL/boost/iterator/transform_iterator.hpp>
@@ -69,6 +72,7 @@
 #include <CGAL/Bbox_3.h>
 
 #include <queue>
+#include <unordered_map>
 
 // for default parameters
 #if defined(CGAL_EIGEN3_ENABLED)
@@ -92,12 +96,20 @@ struct Skel_HDS_vertex_type : public HalfedgeDS_vertex_max_base_with_id<Refs, Po
 };
 
 template <class vertex_descriptor>
-struct Skel_polyhedron_items_3: CGAL::Polyhedron_items_with_id_3 {
+struct Skel_polyhedron_items_3 {
     template < class Refs, class Traits>
     struct Vertex_wrapper {
         typedef typename Traits::Point_3 Point;
       typedef Skel_HDS_vertex_type< Refs, Point, std::size_t, vertex_descriptor> Vertex;
     };
+    template < class Refs, class Traits>
+    struct Halfedge_wrapper {
+        typedef HalfedgeDS_halfedge_max_base_with_id<Refs, std::size_t> Halfedge;
+    };
+    template < class Refs, class Traits>
+    struct Face_wrapper {
+        typedef HalfedgeDS_face_max_base_with_id< Refs, Tag_false, std::size_t>  Face;
+    };
 };
 
 } //end of namespace internal
@@ -193,7 +205,7 @@ public:
   typedef typename Traits::Vector_3                                             Vector;
 
   typedef typename boost::graph_traits<TriangleMesh>::vertex_descriptor  Input_vertex_descriptor;
-  typedef CGAL::Polyhedron_3<Traits,internal::Skel_polyhedron_items_3<Input_vertex_descriptor> > mTriangleMesh;
+  typedef CGAL::HalfedgeDS_default<Traits,internal::Skel_polyhedron_items_3<Input_vertex_descriptor> > mTriangleMesh;
   typedef typename boost::property_map<mTriangleMesh, CGAL::vertex_point_t>::type mVertexPointMap;
   typedef typename boost::property_map<mTriangleMesh, boost::vertex_index_t>::type VertexIndexMap;
   typedef typename boost::property_map<mTriangleMesh, boost::halfedge_index_t>::type HalfedgeIndexMap;
@@ -223,8 +235,12 @@ public:
   typedef typename boost::graph_traits<mTriangleMesh>::edge_descriptor         edge_descriptor;
   typedef typename boost::graph_traits<mTriangleMesh>::edge_iterator           edge_iterator;
 
+  struct Less_id {
+    bool operator()(const edge_descriptor& x, const edge_descriptor& y) const { return x.id() < y.id(); }
+  };
+
   // Get weight from the weight interface.
-  typedef CGAL::Weights::Cotangent_weight<mTriangleMesh>                       Weight_calculator;
+  typedef CGAL::Weights::Cotangent_weight<mTriangleMesh, mVertexPointMap, Traits> Weight_calculator;
 
   typedef internal::Curve_skeleton<mTriangleMesh,
                                    VertexIndexMap,
@@ -258,9 +274,9 @@ private:
   /** Traits class. */
   Traits m_traits;
 
-  /** Controling the velocity of movement and approximation quality. */
+  /** Controlling the velocity of movement and approximation quality. */
   double m_omega_H;
-  /** Controling the smoothness of the medial approximation. */
+  /** Controlling the smoothness of the medial approximation. */
   double m_omega_P;
   /** Edges with length less than `min_edge_length` will be collapsed. */
   double m_min_edge_length;
@@ -372,17 +388,19 @@ public:
   Mean_curvature_flow_skeletonization(const TriangleMesh& tmesh,
                                       VertexPointMap vertex_point_map,
                                       const Traits& traits = Traits())
-    : m_traits(traits), m_weight_calculator(true /* use_clamped_version */)
+    :
+      m_tmesh(),
+      m_tmesh_point_pmap(get(CGAL::vertex_point, m_tmesh)),
+      m_traits(traits),
+      m_weight_calculator(m_tmesh, m_tmesh_point_pmap, m_traits, true /* use_clamped_version */)
   {
     init(tmesh, vertex_point_map);
   }
 
   Mean_curvature_flow_skeletonization(const TriangleMesh& tmesh,
                                       const Traits& traits = Traits())
-    : m_traits(traits), m_weight_calculator(true /* use_clamped_version */)
-  {
-    init(tmesh);
-  }
+    : Mean_curvature_flow_skeletonization(tmesh, get(vertex_point, tmesh), traits)
+  { }
   #endif
   /// @} Constructor
 
@@ -502,6 +520,29 @@ public:
     m_omega_P = value;
   }
 
+  /// \cgalAdvancedFunction
+  /// \cgalAdvancedBegin
+  /// sets the vertices in the range `[begin, end)` as fixed.  Fixed vertices will not be moved
+  /// during contraction and this will therefore prevent convergence
+  /// towards the skeleton if `contract_until_convergence()` is used.
+  /// It is useful only if the end goal is to retrieve the meso-skeleton
+  /// after a number of `contract_geometry()`, keeping the specified
+  /// vertices fixed in place.
+  /// \tparam InputIterator a model of `InputIterator` with `boost::graph_traits<TriangleMesh>::%vertex_descriptor` as value type.
+  /// \cgalAdvancedEnd
+  template<class InputIterator>
+  void set_fixed_vertices(InputIterator begin, InputIterator end)
+  {
+    std::unordered_set<Input_vertex_descriptor> set(begin, end);
+
+    for(vertex_descriptor vd : vertices(m_tmesh))
+    {
+      if (set.find(vd->vertices[0]) != set.end()) {
+        vd->is_fixed = true;
+      }
+    }
+  }
+
   /// \cond SKIP_FROM_MANUAL
   void set_zero_TH(double value)
   {
@@ -729,7 +770,7 @@ public:
     MCFSKEL_DEBUG(print_edges();)
 
     MCFSKEL_INFO(double area = CGAL::Polygon_mesh_processing::area(m_tmesh,
-      CGAL::Polygon_mesh_processing::parameters::vertex_point_map(m_tmesh_point_pmap));)
+      CGAL::parameters::vertex_point_map(m_tmesh_point_pmap));)
     MCFSKEL_INFO(Rcpp::Rcout << "area " << area << "\n";)
   }
 
@@ -753,7 +794,7 @@ public:
       detect_degeneracies();
 
       double area = CGAL::Polygon_mesh_processing::area(m_tmesh,
-        CGAL::Polygon_mesh_processing::parameters::vertex_point_map(m_tmesh_point_pmap)
+        CGAL::parameters::vertex_point_map(m_tmesh_point_pmap)
         .geom_traits(m_traits));
       double area_ratio = fabs(last_area - area) / m_original_area;
 
@@ -832,12 +873,13 @@ private:
   }
 
   /// Initialize some global data structures such as vertex id.
-  void init(const TriangleMesh& tmesh)
+  void init(const TriangleMesh& tmesh, VertexPointMap vpm)
   {
     typedef std::pair<Input_vertex_descriptor, vertex_descriptor> Vertex_pair;
     std::vector<Vertex_pair> v2v;
     copy_face_graph(tmesh, m_tmesh,
-                    CGAL::parameters::vertex_to_vertex_output_iterator(std::back_inserter(v2v)));
+                    CGAL::parameters::vertex_to_vertex_output_iterator(std::back_inserter(v2v))
+                                     .vertex_point_map(vpm));
 
     // copy input vertices to keep correspondence
     for(const Vertex_pair& vp : v2v)
@@ -858,7 +900,7 @@ private:
     m_are_poles_computed = false;
 
     m_original_area = CGAL::Polygon_mesh_processing::area(m_tmesh,
-      CGAL::Polygon_mesh_processing::parameters::vertex_point_map(m_tmesh_point_pmap)
+      CGAL::parameters::vertex_point_map(m_tmesh_point_pmap)
       .geom_traits(m_traits));
 
     m_vertex_id_count = static_cast<int>(num_vertices(m_tmesh));
@@ -884,11 +926,9 @@ private:
   void compute_edge_weight()
   {
     m_edge_weight.clear();
-    m_edge_weight.reserve(2 * num_edges(m_tmesh));
+    m_edge_weight.reserve(num_halfedges(m_tmesh));
     for(halfedge_descriptor hd : halfedges(m_tmesh))
-    {
-      m_edge_weight.push_back(m_weight_calculator(hd, m_tmesh, m_tmesh_point_pmap));
-    }
+      m_edge_weight.push_back(m_weight_calculator(hd));
   }
 
   /// Assemble the left hand side.
@@ -1194,7 +1234,7 @@ private:
       vertex_descriptor vk = target(ek, m_tmesh);
 
       // split the edge
-      halfedge_descriptor en = m_tmesh.split_edge(ei);
+      halfedge_descriptor en = Euler::split_edge(ei, m_tmesh);
       // split the incident faces
       Euler::split_face(en, next(ei,m_tmesh), m_tmesh);
       if (! is_border(ej,m_tmesh))
@@ -1340,9 +1380,9 @@ private:
   {
     namespace PMP = CGAL::Polygon_mesh_processing;
 
-    boost::unordered_map<face_descriptor, Vector> normals;
+    std::unordered_map<face_descriptor, Vector> normals;
     boost::associative_property_map<
-      boost::unordered_map<face_descriptor, Vector> > normals_pmap(normals);
+      std::unordered_map<face_descriptor, Vector> > normals_pmap(normals);
     PMP::compute_face_normals(m_tmesh, normals_pmap);
 
     m_normals.resize(num_vertices(m_tmesh));
@@ -1352,7 +1392,7 @@ private:
       int vid = static_cast<int>(get(m_vertex_id_pmap, v));
       m_normals[vid] = PMP::compute_vertex_normal(v
                           , m_tmesh
-                          , PMP::parameters::geom_traits(m_traits)
+                          , CGAL::parameters::geom_traits(m_traits)
                           .face_normal_map(normals_pmap));
     }
   }
@@ -1390,7 +1430,7 @@ std::size_t Mean_curvature_flow_skeletonization<TriangleMesh, Traits_, VertexPoi
 {
   std::size_t cnt=0, prev_cnt=0;
 
-  std::set<edge_descriptor> edges_to_collapse, non_topologically_valid_collapses;
+  std::set<edge_descriptor,Less_id> edges_to_collapse, non_topologically_valid_collapses;
 
   for(edge_descriptor ed : edges(m_tmesh))
     if ( edge_should_be_collapsed(ed) )
@@ -1457,3 +1497,4 @@ std::size_t Mean_curvature_flow_skeletonization<TriangleMesh, Traits_, VertexPoi
 } //namespace CGAL
 
 #endif // CGAL_MEAN_CURVATURE_FLOW_SKELETONIZATION_H
+
