@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/Gaussian_map.h $
-// $Id: Gaussian_map.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/Minkowski_sum_3/include/CGAL/Minkowski_sum_3/Gaussian_map.h $
+// $Id: Gaussian_map.h 3674c93 2022-11-15T15:21:01+01:00 albert-github
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -181,7 +181,7 @@ class Gaussian_map :
         CGAL_NEF_TRACEN( "first+current:" << first << "+" << current );
         typename Nef_polyhedron_3::SHalfedge_around_sface_const_circulator sfc(sec), send(sfc);
         CGAL_For_all(sfc, send) {
-          CGAL_NEF_TRACEN( "sedge->cirlce() " << sfc->circle() );
+          CGAL_NEF_TRACEN( "sedge->circle() " << sfc->circle() );
           if(sfc->circle() != current) {
             if(sfc->circle() != first)
               ++circles;
@@ -1227,3 +1227,4 @@ std::ostream& operator<<(std::ostream& out, const CGAL::Gaussian_map<Kernel, Nef
 
 } //namespace CGAL
 #endif // CGAL_MS3_GAUSSIAN_MAP
+
