@@ -5,8 +5,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/Poisson_surface_reconstruction_3/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h $
-// $Id: Poisson_implicit_surface_oracle_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/Poisson_surface_reconstruction_3/include/CGAL/Surface_mesher/Poisson_implicit_surface_oracle_3.h $
+// $Id: Poisson_implicit_surface_oracle_3.h ce65fa9 2022-09-28T11:53:10+01:00 Andreas Fabri
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -47,46 +47,6 @@ namespace CGAL {
 
   namespace Surface_mesher {
 
-    /**
-       Type and functions required in GT:
-         - Compute_scalar_product_3             (from rev. 29646)
-         - Compute_squared_distance_3
-         - Compute_squared_radius_3
-         - Construct_center_3
-         - Construct_midpoint_3
-         - Construct_point_on_3                 (from rev. 29646)
-         - Construct_scaled_vector_3
-         - Construct_segment_3                  (from rev. 29646)
-         - Construct_translated_point_3
-         - Construct_vector_3
-         - FT
-         - Has_on_bounded_side_3                (from rev. 29646)
-         - Line_3
-         - Point_3
-         - Ray_3
-         - Segment_3
-         - Sphere_3
-         - Vector_3                             (from rev. 29646)
-         - compute_scalar_product_3_object      (from rev. 29646)
-         - compute_squared_distance_3_object
-         - compute_squared_radius_3_object
-         - construct_center_3_object
-         - construct_line_3_object              (no longer, since rev. 29646)
-         - construct_midpoint_3_object
-         - construct_point_on_3_object          (from rev. 29646)
-         - construct_scaled_vector_3_object
-         - construct_segment_3_object           (from rev. 29646)
-         - construct_translated_point_3_object
-         - construct_vector_3_object
-         - has_on_bounded_side_3_object         (from rev. 29646)
-(Computed by use of:
- perl -ne '/GT\(\)\.([a-zA-Z_0-9]+)/
-             && print "$1\n";' {implicit_surface_oracle_3.h,Sphere_oracle_3.h} | sort -u
- perl -ne '/GT::([a-zA-Z_0-9]+)/
-             && print "$1\n";' {implicit_surface_oracle_3.h,Sphere_oracle_3.h} | sort -u
-)
-    */
-
 
   template <
     class GT,
@@ -502,3 +462,4 @@ FT approximate_sqrt(const FT x) {
 
 
 #endif  // CGAL_SURFACE_MESHER_POISSON_IMPLICIT_SURFACE_ORACLE_3_H
+
