@@ -4,8 +4,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.4/Tetrahedral_remeshing/include/CGAL/tetrahedral_remeshing.h $
-// $Id: tetrahedral_remeshing.h bfd4e99 2020-09-15T15:42:24+02:00 Jane Tournois
+// $URL: https://github.com/CGAL/cgal/blob/v5.6/Tetrahedral_remeshing/include/CGAL/tetrahedral_remeshing.h $
+// $Id: tetrahedral_remeshing.h 1b6d590 2022-12-14T10:18:16+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -24,9 +24,11 @@
 #include <CGAL/Tetrahedral_remeshing/internal/tetrahedral_adaptive_remeshing_impl.h>
 #include <CGAL/Tetrahedral_remeshing/internal/compute_c3t3_statistics.h>
 
-#include <CGAL/boost/graph/Named_function_parameters.h>
+#include <CGAL/Named_function_parameters.h>
 #include <CGAL/boost/graph/named_params_helper.h>
 
+#include <CGAL/property_map.h>
+
 #ifdef CGAL_DUMP_REMESHING_STEPS
 #include <sstream>
 #endif
@@ -160,11 +162,11 @@ namespace CGAL
 * @todo implement non-uniform sizing field instead of uniform target edge length
 */
 template<typename Traits, typename TDS, typename SLDS,
-         typename NamedParameters>
+         typename NamedParameters = parameters::Default_named_parameters>
 void tetrahedral_isotropic_remeshing(
   CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
   const double& target_edge_length,
-  const NamedParameters& np)
+  const NamedParameters& np = parameters::default_values())
 {
   typedef CGAL::Triangulation_3<Traits, TDS, SLDS> Triangulation;
   tetrahedral_isotropic_remeshing(
@@ -175,11 +177,11 @@ void tetrahedral_isotropic_remeshing(
 }
 
 template<typename Traits, typename TDS, typename SLDS,
-         typename NamedParameters>
+         typename NamedParameters = parameters::Default_named_parameters>
 void tetrahedral_isotropic_remeshing(
   CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
   const float& target_edge_length,
-  const NamedParameters& np)
+  const NamedParameters& np = parameters::default_values())
 {
   typedef CGAL::Triangulation_3<Traits, TDS, SLDS> Triangulation;
   tetrahedral_isotropic_remeshing(
@@ -197,7 +199,7 @@ void tetrahedral_isotropic_remeshing(
   const SizingFunction& sizing,
   const NamedParameters& np)
 {
-  CGAL_assertion(tr.is_valid(true));
+  CGAL_assertion(tr.is_valid());
 
   typedef CGAL::Triangulation_3<Traits, TDS, SLDS> Tr;
 
@@ -225,24 +227,22 @@ void tetrahedral_isotropic_remeshing(
                        Tetrahedral_remeshing::internal::All_cells_selected<Tr>());
 
   typedef std::pair<typename Tr::Vertex_handle, typename Tr::Vertex_handle> Edge_vv;
-  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Edge_vv> No_edge;
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::edge_is_constrained_t,
     NamedParameters,
-    No_edge//default
+    Constant_property_map<Edge_vv, bool>//default
   > ::type ECMap;
   ECMap ecmap = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
-                                 No_edge());
+                                 Constant_property_map<Edge_vv, bool>(false));
 
   typedef typename Tr::Facet Facet;
-  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Facet> No_facet;
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::facet_is_constrained_t,
     NamedParameters,
-    No_facet//default
+    Constant_property_map<Facet, bool>//default
   > ::type FCMap;
   FCMap fcmap = choose_parameter(get_parameter(np, internal_np::facet_is_constrained),
-                                 No_facet());
+                                 Constant_property_map<Facet, bool>(false));
 
   typedef typename internal_np::Lookup_named_param_def <
     internal_np::visitor_t,
@@ -273,6 +273,8 @@ void tetrahedral_isotropic_remeshing(
 
 #ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
   Rcpp::Rcout << "done." << std::endl;
+#endif
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
   Tetrahedral_remeshing::internal::compute_statistics(
     remesher.tr(), cell_select, "statistics_begin.txt");
 #endif
@@ -281,7 +283,7 @@ void tetrahedral_isotropic_remeshing(
   std::size_t nb_extra_iterations = 3;
   remesher.remesh(max_it, nb_extra_iterations);
 
-#ifdef CGAL_TETRAHEDRAL_REMESHING_VERBOSE
+#ifdef CGAL_TETRAHEDRAL_REMESHING_DEBUG
   const double angle_bound = 5.0;
   Tetrahedral_remeshing::debug::dump_cells_with_small_dihedral_angle(tr,
     angle_bound, cell_select, "bad_cells.mesh");
@@ -290,15 +292,6 @@ void tetrahedral_isotropic_remeshing(
 #endif
 }
 
-template<typename Traits, typename TDS, typename SLDS>
-void tetrahedral_isotropic_remeshing(
-  CGAL::Triangulation_3<Traits, TDS, SLDS>& tr,
-  const double& target_edge_length)
-{
-  tetrahedral_isotropic_remeshing(tr, target_edge_length,
-                                 CGAL::parameters::all_default());
-}
-
 /*!
 * \ingroup PkgTetrahedralRemeshingRef
 * converts the triangulation contained in the input to a `Triangulation_3`.
@@ -349,11 +342,11 @@ convert_to_triangulation_3(
 ///////
 template<typename Tr,
          typename CornerIndex, typename CurveIndex,
-         typename NamedParameters>
+         typename NamedParameters = parameters::Default_named_parameters>
 void tetrahedral_isotropic_remeshing(
   CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
   const double& target_edge_length,
-  const NamedParameters& np)
+  const NamedParameters& np = parameters::default_values())
 {
   tetrahedral_isotropic_remeshing(
     c3t3,
@@ -364,11 +357,11 @@ void tetrahedral_isotropic_remeshing(
 
 template<typename Tr,
          typename CornerIndex, typename CurveIndex,
-         typename NamedParameters>
+         typename NamedParameters = parameters::Default_named_parameters>
 void tetrahedral_isotropic_remeshing(
   CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
   const float& target_edge_length,
-  const NamedParameters& np)
+  const NamedParameters& np = parameters::default_values)
 {
   tetrahedral_isotropic_remeshing(
     c3t3,
@@ -377,26 +370,16 @@ void tetrahedral_isotropic_remeshing(
     np);
 }
 
-template<typename Tr,
-         typename CornerIndex, typename CurveIndex>
-void tetrahedral_isotropic_remeshing(
-  CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
-  const double& target_edge_length)
-{
-  return tetrahedral_isotropic_remeshing(c3t3, target_edge_length,
-                                        CGAL::parameters::all_default());
-}
-
 template<typename Tr,
          typename CornerIndex, typename CurveIndex,
          typename SizingFunction,
-         typename NamedParameters>
+         typename NamedParameters = parameters::Default_named_parameters>
 void tetrahedral_isotropic_remeshing(
   CGAL::Mesh_complex_3_in_triangulation_3<Tr, CornerIndex, CurveIndex>& c3t3,
   const SizingFunction& sizing,
-  const NamedParameters& np)
+  const NamedParameters& np = parameters::default_values())
 {
-  CGAL_assertion(c3t3.triangulation().tds().is_valid(true));
+  CGAL_assertion(c3t3.triangulation().tds().is_valid());
 
   using parameters::get_parameter;
   using parameters::choose_parameter;
@@ -410,34 +393,33 @@ void tetrahedral_isotropic_remeshing(
     = choose_parameter(get_parameter(np, internal_np::smooth_constrained_edges),
       false);
 
+  typedef typename Tr::Cell_handle Cell_handle;
   typedef typename internal_np::Lookup_named_param_def <
   internal_np::cell_selector_t,
               NamedParameters,
-              Tetrahedral_remeshing::internal::All_cells_selected<Tr>//default
+              Constant_property_map<Cell_handle, bool>//default
               > ::type SelectionFunctor;
   SelectionFunctor cell_select
     = choose_parameter(get_parameter(np, internal_np::cell_selector),
-                       Tetrahedral_remeshing::internal::All_cells_selected<Tr>());
+                       Constant_property_map<Cell_handle, bool>(true));
 
   typedef std::pair<typename Tr::Vertex_handle, typename Tr::Vertex_handle> Edge_vv;
-  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Edge_vv> No_edge;
   typedef typename internal_np::Lookup_named_param_def <
   internal_np::edge_is_constrained_t,
               NamedParameters,
-              No_edge//default
+              Constant_property_map<Edge_vv, bool>//default
               > ::type ECMap;
   ECMap ecmap = choose_parameter(get_parameter(np, internal_np::edge_is_constrained),
-                                 No_edge());
+                                 Constant_property_map<Edge_vv, bool>(false));
 
   typedef typename Tr::Facet Facet;
-  typedef Tetrahedral_remeshing::internal::No_constraint_pmap<Facet> No_facet;
   typedef typename internal_np::Lookup_named_param_def <
   internal_np::facet_is_constrained_t,
               NamedParameters,
-              No_facet//default
+              Constant_property_map<Facet, bool>//default
               > ::type FCMap;
   FCMap fcmap = choose_parameter(get_parameter(np, internal_np::facet_is_constrained),
-                                 No_facet());
+                                 Constant_property_map<Facet, bool>(false));
 
   typedef typename internal_np::Lookup_named_param_def <
               internal_np::visitor_t,
@@ -496,3 +478,4 @@ void tetrahedral_isotropic_remeshing(
 }//end namespace CGAL
 
 #endif //TETRAHEDRAL_REMESHING_H
+
